package com.company.defaultroles.security;

import com.company.defaultroles.entity.User;
import io.jmix.security.authentication.RoleGrantedAuthority;
import io.jmix.security.model.ResourceRole;
import io.jmix.security.role.ResourceRoleRepository;
import io.jmix.securitydata.user.AbstractDatabaseUserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Primary
@Component("UserRepository")
public class DatabaseUserRepository extends AbstractDatabaseUserRepository<User> {

    @Autowired
    private ResourceRoleRepository resourceRoleRepository;

    @Override
    protected Class<User> getUserClass() {
        return User.class;
    }

    @Override
    protected void initSystemUser(User systemUser) {
        Collection<GrantedAuthority> authorities = getGrantedAuthoritiesBuilder()
                .addResourceRole(FullAccessRole.CODE)
                .build();
        systemUser.setAuthorities(authorities);
    }

    @Override
    protected void initAnonymousUser(User anonymousUser) {
    }

    @Override
    public User loadUserByUsername(String username) throws UsernameNotFoundException {
        User user = super.loadUserByUsername(username);
        Collection<GrantedAuthority> authorities = new ArrayList<>(user.getAuthorities());
        Collection<ResourceRole> defaultRoles = getDefaultRoles();
        for (ResourceRole defaultRole : defaultRoles) {
            authorities.add(RoleGrantedAuthority.ofResourceRole(defaultRole));
        }
        user.setAuthorities(authorities);
        return user;
    }

    private Collection<ResourceRole> getDefaultRoles() {
        List<ResourceRole> defaultRoles = new ArrayList<>();
        ResourceRole customerRole = resourceRoleRepository.getRoleByCode(CustomerRole.CODE);
        defaultRoles.add(customerRole);
        return defaultRoles;
    }
}